// Copyright (C) 1999 Jean-Marc Valin

#include "BufferedNode.h"
#include "Buffer.h"
#include "Vector.h"
#include <stdlib.h>

class CAllPole;

DECLARE_NODE(CAllPole)
/*Node

 * @name CAllPole
 * @category Signal:DSP
 * @description No description available

 * @input_name INPUT
 * @input_description No description available

 * @input_name FILTER
 * @input_description No description available

 * @output_name OUTPUT
 * @output_description No description available

END*/


class CAllPole : public BufferedNode {
   
   int inputID;
   int outputID;
   int filterID;

public:
   CAllPole(string nodeName, ParameterSet params)
   : BufferedNode(nodeName, params)
   {
      inputID = addInput("INPUT");
      filterID = addInput("FILTER");
      outputID = addOutput("OUTPUT");
      
   }

   virtual void specificInitialize()
   {
      outputs[outputID].lookBack += 1;
      this->BufferedNode::specificInitialize();
   }

   void calculate(int output_id, int count, Buffer &out)
   {
      ObjectRef inputValue = getInput(inputID, count);
      ObjectRef filterValue = getInput(filterID, count);

      if (inputValue->status != Object::valid)
      {
	 out[count] = inputValue;
         return;
      }

      const Vector<float> &in = object_cast<Vector<float> > (inputValue);
      int inputLength = in.size();

      Vector<float> &output = *Vector<float>::alloc(inputLength);
      out[count] = &output;

      for (int i=0;i<inputLength;i++)
         output[i]=in[i];

      if (filterValue->status != Object::valid)
      {
         return;
      }

      const Vector<float> &filter = object_cast<Vector<float> > (filterValue);
      const Vector<float> *past;
      bool can_look_back = false;
      if (count > 0)   
      {
         ObjectRef pastInputValue = this->getOutput(outputID, count-1);
         if (pastInputValue->status == Object::valid)
         {
            can_look_back=true;
            past = &object_cast<Vector<float> > (pastInputValue);
         }      
      }      
      

      int size = filter.size();

      if (can_look_back)
      {
         for (int i=0;i<size-1;i++)
         {
            int j, k;
            for (j=inputLength-1, k=1+i; k<size ; j--, k++)
            {
               output[i]-=(*past)[j]*filter[k];
            }         
         }
      }
      
      for (int i=0;i<inputLength;i++)
      {
         int j, k;
         for (j=i-1, k=1;j>=max(0,i-size+1) ; j--, k++)
            output[i]-=output[j]*filter[k];
      }
      for (int i=0;i<inputLength;i++)
         output[i]/=filter[0];
   }

};
